@@ -18,9 +18,10 @@ from ..dynamics.configure_problem import ConfigureProblem
 from ..gui.plot import CustomPlot, PlotOcp
 from ..gui.graph import OcpToConsole, OcpToGraph
 from ..interfaces.biorbd_interface import BiorbdInterface
-from ..interfaces.SolverOptions import Solver
+from ..interfaces.solver_options import Solver
 from ..limits.constraints import ConstraintFunction, ConstraintFcn, ConstraintList, Constraint, ContinuityFunctions
 from ..limits.phase_transition import PhaseTransitionList
+from ..limits.multinode_constraint import MultinodeConstraintList
 from ..limits.objective_functions import ObjectiveFcn, ObjectiveList, Objective
 from ..limits.path_conditions import BoundsList, Bounds
 from ..limits.path_conditions import InitialGuess, InitialGuessList
@@ -34,7 +35,7 @@ from ..misc.utils import check_version
 from ..optimization.parameters import ParameterList, Parameter
 from ..optimization.solution import Solution
 
-check_version(biorbd, "1.8.0", "1.9.0")
+check_version(biorbd, "1.8.4", "1.9.0")
 
 
 class OptimalControlProgram:
@@ -50,6 +51,8 @@ class OptimalControlProgram:
         Constraints that are not phase dependent (mostly parameters and continuity constraints)
     g_internal: list[list[Constraint]]
         All the constraints internally defined by the OCP at each of the node of the phase
+    g_implicit: list[list[Constraint]]
+        All the implicit constraints defined by the OCP at each of the node of the phase
     J: list
         Objective values that are not phase dependent (mostly parameters)
     isdef_x_init: bool
@@ -139,6 +142,7 @@ class OptimalControlProgram:
         variable_mappings: BiMappingList = None,
         plot_mappings: Mapping = None,
         phase_transitions: PhaseTransitionList = None,
+        multinode_constraints: MultinodeConstraintList = None,
         n_threads: int = 1,
         use_sx: bool = False,
         skip_continuity: bool = False,
@@ -226,6 +230,7 @@ class OptimalControlProgram:
             "variable_mappings": variable_mappings,
             "plot_mappings": plot_mappings,
             "phase_transitions": phase_transitions,
+            "multinode_constraints": multinode_constraints,
             "n_threads": n_threads,
             "use_sx": use_sx,
         }
@@ -294,6 +299,8 @@ class OptimalControlProgram:
         elif not isinstance(objective_functions, ObjectiveList):
             raise RuntimeError("objective_functions should be built from an Objective or ObjectiveList")
 
+        self.implicit_constraints = ConstraintList()
+
         if constraints is None:
             constraints = ConstraintList()
         elif isinstance(constraints, Constraint):
@@ -313,6 +320,11 @@ class OptimalControlProgram:
         elif not isinstance(phase_transitions, PhaseTransitionList):
             raise RuntimeError("phase_transitions should be built from an PhaseTransitionList")
 
+        if multinode_constraints is None:
+            multinode_constraints = MultinodeConstraintList()
+        elif not isinstance(multinode_constraints, MultinodeConstraintList):
+            raise RuntimeError("multinode_constraints should be built from an MultinodeConstraintList")
+
         if ode_solver is None:
             ode_solver = OdeSolver.RK4()
         elif not isinstance(ode_solver, OdeSolverBase):
@@ -333,6 +345,8 @@ class OptimalControlProgram:
         self.J_internal = []
         self.g = []
         self.g_internal = []
+        self.g_implicit = []
+
         self.v = OptimizationVector(self)
 
         # nlp is the core of a phase
@@ -397,7 +411,7 @@ class OptimalControlProgram:
         # Prepare phase transitions (Reminder, it is important that parameters are declared before,
         # otherwise they will erase the phase_transitions)
         self.phase_transitions = phase_transitions.prepare_phase_transitions(self)
-
+        self.multinode_constraints = multinode_constraints.prepare_multinode_constraints(self)
         # Skipping creates a valid but unsolvable OCP class
         if not skip_continuity:
             # Inner- and inter-phase continuity
@@ -412,6 +426,7 @@ class OptimalControlProgram:
         self.update_initial_guess(x_init, u_init)
 
         # Prepare constraints
+        self.update_constraints(self.implicit_constraints)
         self.update_constraints(constraints)
 
         # Prepare objectives
@@ -639,9 +654,11 @@ class OptimalControlProgram:
                 if cost_type == CostType.OBJECTIVES:
                     penalties = nlp.J
                     penalties_internal = nlp.J_internal
+                    penalties_implicit = []
                 else:  # Constraints
                     penalties = nlp.g
                     penalties_internal = nlp.g_internal
+                    penalties_implicit = nlp.g_implicit
 
                 for penalty in penalties:
                     if not penalty:
@@ -651,6 +668,10 @@ class OptimalControlProgram:
                     if not penalty_internal:
                         continue
                     name_unique_objective.append(penalty_internal.name)
+                for penalty_implicit in penalties_implicit:
+                    if not penalty_implicit:
+                        continue
+                    name_unique_objective.append(penalty_implicit.name)
             color = {}
             for i, name in enumerate(name_unique_objective):
                 color[name] = plt.cm.viridis(i / len(name_unique_objective))
@@ -676,7 +697,11 @@ class OptimalControlProgram:
 
             out = []
             if penalty.transition:
-                raise NotImplementedError("add_plot_penalty with phase transition is not implemented yet")
+                out.append(
+                    penalty.weighted_function_non_threaded(
+                        x.reshape((-1, 1)), u.reshape((-1, 1)), p, penalty.weight, _target, dt
+                    )
+                )
             elif penalty.derivative or penalty.explicit_derivative:
                 out.append(penalty.weighted_function_non_threaded(x[:, [0, -1]], u, p, penalty.weight, _target, dt))
             else:
@@ -730,9 +755,11 @@ class OptimalControlProgram:
             if cost_type == CostType.OBJECTIVES:
                 penalties = nlp.J
                 penalties_internal = nlp.J_internal
+                penalties_implicit = []
             elif cost_type == CostType.CONSTRAINTS:
                 penalties = nlp.g
                 penalties_internal = nlp.g_internal
+                penalties_implicit = nlp.g_implicit
             elif cost_type == CostType.ALL:
                 self.add_plot_penalty(CostType.OBJECTIVES)
                 self.add_plot_penalty(CostType.CONSTRAINTS)
@@ -742,6 +769,7 @@ class OptimalControlProgram:
 
             add_penalty(penalties)
             add_penalty(penalties_internal)
+            add_penalty(penalties_implicit)
         return
 
     def prepare_plots(
@@ -780,7 +808,7 @@ class OptimalControlProgram:
 
     def solve(
         self,
-        solver: Solver.Generic = None,
+        solver: Union[Solver, Solver.Generic] = None,
         warm_start: Solution = None,
     ) -> Solution:
         """
